﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Text.RegularExpressions;

namespace PeerBlockListUnblocker
{
	public partial class FormMain : Form
	{
		Regex hashSearch;
		Regex uidValidator;
		bool atValidDirectory = false;
		const string PeerBlockConfigFile = @"/peerblock.conf";
		

		public FormMain()
		{
			InitializeComponent();
			hashSearch = new Regex(@"<UniqueId>([0-9a-f]|-){36}</UniqueId>", RegexOptions.Compiled);
			uidValidator = new Regex(@"([0-9a-f]){8}-(([0-9a-f]){4}-){3}([0-9a-f]){12}", RegexOptions.Compiled);
			textBoxFolderLocation_TextChanged(textBoxFolderLocation, null);

			//351,17
			ResizeTextBox(textBoxOldUID, 351, 17);
			ResizeTextBox(textBoxNewUID, 351, 17);
		}

		private void ResizeTextBox(TextBox textBox, int width, int height)
		{
			textBox.Multiline = true;
			textBox.MinimumSize = new Size(width, height);
			textBox.MaximumSize = new Size(width, height);
			textBox.Size = new Size(width, height);
			textBox.Multiline = false;
		}

		private void buttonSearch_Click(object sender, EventArgs e)
		{
			folderBrowserDialog.ShowDialog();
			textBoxFolderLocation.Text = folderBrowserDialog.SelectedPath;
		}

		private void textBoxFolderLocation_TextChanged(object sender, EventArgs e)
		{
			var textBox = (TextBox)sender;

			if (Directory.Exists(textBox.Text) && File.Exists(textBox.Text + PeerBlockConfigFile))
			{
				atValidDirectory = true;
				string file = textBox.Text + PeerBlockConfigFile;
				var match = hashSearch.Match(File.ReadAllText(file));
				const int hashSize = 36; // inc. dashes
				const int tagSize = 16; // 8 * 2
				const int symSize = 5; // <></>

				if (match.Value.Length == hashSize + tagSize + symSize)
				{
					string hash = match.Value.Substring(10, 36);
					textBoxOldUID.Text = hash;
				}
			}
			else
			{
				atValidDirectory = false;
				textBoxOldUID.Text = "";
			}
		}

		

		private void buttonGenerateSet_Click(object sender, EventArgs e)
		{
			textBoxNewUID.Text = InsertUIDDashes(GenerateUID());
			TrySetNewUID(sender, null);
		}

		private void TrySetNewUID(object sender, EventArgs e)
		{
			if (!atValidDirectory) return;

			string uid = textBoxNewUID.Text;

			if (!ValidateUID(uid))
			{
				MessageBox.Show("Input Unique ID is invalid!", "Invalid UID", MessageBoxButtons.OK);
				return;
			}

			ReplaceUID(textBoxFolderLocation.Text + PeerBlockConfigFile);

			textBoxOldUID.Text = uid;
		}

		private void textBoxNewUID_KeyDown(object sender, KeyEventArgs e)
		{
			// this list is surprisingly well thought out
			if ((e.KeyData < Keys.D0 || e.KeyData > Keys.F) && // 0-9 a-f
				(e.KeyData != Keys.Subtract && e.KeyData != Keys.OemMinus && // dashes
				 e.KeyData != Keys.Back && e.KeyData != Keys.Delete && // backspace and delete
				 e.KeyData != Keys.Left && e.KeyData != Keys.Right && // left and right treading
				 e.KeyData != (Keys.Control | Keys.V) && e.KeyData != (Keys.Control | Keys.C))) // copy and paste
				e.SuppressKeyPress = true;
		}

		private bool ValidateUID(string uid)
		{
			return uidValidator.IsMatch(uid);
		}

		private string GenerateUID()
		{
			var rand = new Random();
			byte[] bytes = new byte[64];
			for (int i = 0; i < 64; i++)
			{
				bytes[i] = (byte)(rand.Next(0, 0xFF) & 0xFF);
			}
			// MD5 is 16 bytes, so it's perfect
			return BytesToStringHex(new System.Security.Cryptography.MD5Cng().ComputeHash(bytes));
		}

		private string InsertUIDDashes(string uid)
		{
			return String.Format("{0}-{1}-{2}-{3}-{4}",
				uid.Substring(0, 8),
				uid.Substring(8, 4),
				uid.Substring(12, 4),
				uid.Substring(16, 4),
				uid.Substring(20, 12));
		}

		private string BytesToStringHex(byte[] bytes)
		{
			var sb = new char[bytes.Length * 2];

			for (int i = 0; i < bytes.Length; i++)
			{
				byte b = bytes[i];

				sb[i * 2] = (char)(((b >> 4) & 0xF) < 10 ?
					(((b >> 4) & 0xF) + 48) : (((b >> 4) & 0xF) + 87));
				sb[i * 2 + 1] = (char)((b & 0xF) < 10 ? ((b & 0xF) + 48) : ((b & 0xF) + 87));
			}

			return new string(sb);
		}

		protected void ReplaceUID(string path)
		{
			using (FileStream stream = new FileStream(path, FileMode.Open, FileAccess.ReadWrite))
			{
				using (StreamReader reader = new StreamReader(stream))
				{
					long streamPosition = 0L;

					while (!reader.EndOfStream)
					{
						long position = stream.Position;
						string line = reader.ReadLine();
						streamPosition += line.Length + 2; // assuming CR+LF
						if (hashSearch.IsMatch(line))
						{
							string newText = line.Replace(textBoxOldUID.Text, textBoxNewUID.Text);
							stream.Position = streamPosition - line.Length - 2;
							var bytes = CharArrayToByteArray(newText.ToCharArray());
							stream.Write(bytes, 0, bytes.Length);
							stream.Position = stream.Length;
						}
					}

					stream.Position = 0L;
				}
			}
		}

		private byte[] CharArrayToByteArray(char[] array)
		{
			byte[] retArray = new byte[array.Length];
			for (int i = 0; i < array.Length; i++)
				retArray[i] = (byte)array[i];
			return retArray;
		}
	}
}
